import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';

import 'view_state_list_model.dart';

/// 基于
abstract class ViewStateRefreshListModel<T> extends ViewStateListModel<T> {

  ///自定义第一页页数
  int firstPage();

  ///自定义一页数量
  int pageSize();

  EasyRefreshController _refreshController = EasyRefreshController();

  EasyRefreshController get refreshController => _refreshController;

  /// 当前页码
  int _currentPageNum;

  /// 下拉刷新
  ///
  /// [init] 是否是第一次加载
  /// true:  Error时,需要跳转页面
  /// false: Error时,不需要跳转页面,直接给出提示
  Future<List<T>> refresh({bool init = false}) async {
    try {
      _currentPageNum = firstPage();
      var data = await loadData(pageNum: firstPage());
      if (data.isEmpty) {
        refreshController.finishLoad(noMore: true);
        refreshController.resetRefreshState();
        list.clear();
        setEmpty();
      } else {
        onCompleted(data);
        list.clear();
        list.addAll(data);
        // 小于分页的数量,禁止上拉加载更多
        if (data.length < pageSize()) {
          refreshController.finishRefresh();
          refreshController.resetRefreshState();
        } else {
          //防止上次上拉加载更多失败,需要重置状态
          refreshController.finishRefresh();
          refreshController.resetRefreshState();
          refreshController.resetLoadState();
        }
        setIdle();
      }
      return data;
    } catch (e, s) {
      /// 页面已经加载了数据,如果刷新报错,不应该直接跳转错误页面
      /// 而是显示之前的页面数据.给出错误提示
      if (init) list.clear();
      refreshController.finishRefresh(success: false);
      setError(e, s);
      return null;
    }
  }

  /// 上拉加载更多
  Future<List<T>> loadMore() async {
    try {
      var data = await loadData(pageNum: ++_currentPageNum);
      if (data.isEmpty) {
        _currentPageNum--;
        refreshController.finishLoad(noMore: true);
      } else {
        onCompleted(data);
        list.addAll(data);
        if (data.length < pageSize()) {
          refreshController.finishLoad(noMore: true);
        } else {
          refreshController.finishLoad(noMore: false);
        }
        notifyListeners();
      }
      return data;
    } catch (e, s) {
      _currentPageNum--;
      refreshController.finishLoad(success: false, noMore: true);
      debugPrint('error--->\n' + e.toString());
      debugPrint('statck--->\n' + s.toString());
      return null;
    }
  }

  // 加载数据
  Future<List<T>> loadData({int pageNum});

  @override
  void dispose() {
    _refreshController.dispose();
    super.dispose();
  }
}
